--imports
local assert = require("justo.assert")
local justo = require("justo")
local suite, test, init = justo.suite, justo.test, justo.init
local Trans = require("dogma.trans.js.Trans")
local Parser = require("dogma.syn.Parser")

--Suite.
return suite("dogma.trans.js._.DirectiveTrans", function()
  local trans, parser

  ----------
  -- init --
  ----------
  init("*", function()
    parser = Parser.new()
    trans = Trans.new()
    trans:transform(parser)
  end):title("Create transformer")

  ---------
  -- env --
  ---------
  test("env", function()
    parser:parse("#!env")
    assert(trans:next()):eq('#!/usr/bin/env node;\n\n')
  end)

  -----------
  -- react --
  -----------
  test("react", function()
    parser:parse("#!react")
    assert(trans:next()):eq('import React, {Component} from "react";\n\n')
  end)

  --------------
  -- coverage --
  --------------
  suite("cov", function()
    test("cov ignore", function()
      parser:parse("#!cov ignore")
      assert(trans:next()):eq('/*istanbul ignore next*/\n')
    end)

    test("coverage ignore next", function()
      parser:parse("#!coverage ignore next")
      assert(trans:next()):eq('/*istanbul ignore next*/\n')
    end)

    test("coverage next expression", function()
      parser:parse("#!coverage ignore next\nx + 1")
      assert(trans:next()):eq('/*istanbul ignore next*/(x+1);\n')
    end)

    test("coverage into loop #1", function()
      parser:parse("while true do\n  #!coverage ignore next\n  x + 1")
      assert(trans:next()):eq('while (true) {/*istanbul ignore next*/(x+1);}\n')
    end)

    test("coverage into loop #2", function()
      parser:parse("while true do\n  #!coverage ignore next")
      assert(trans:next()):eq('while (true) {/*istanbul ignore next*/}\n')
    end)
  end)

  --------
  -- if --
  --------
  suite("if", function()
    suite("custom directive", function()
      init("*", function()
        parser = Parser.new()
        trans = Trans.new({directives={"one"}})
        trans:transform(parser)
      end):title("Define directives")

      test("if", function()
        parser:parse([[
  #!if one then
  print("ok!")
  #!else
  print("wrong!")
  #!end
  ]])
        assert(trans:next()):eq('print("ok!");\n')
      end)

      test("if not", function()
        parser:parse([[
  #!if not two then
  print("ok!")
  #!else
  print("wrong!")
  #!end
  ]])
        assert(trans:next()):eq('print("ok!");\n')
      end)

      test("else", function()
        parser:parse([[
  #!if two then
  print("ok!")
  #!else
  print("wrong!")
  #!end
  ]])
        assert(trans:next()):eq('print("wrong!");\n')
      end)
    end)

    test("if js then - with else", function()
      parser:parse([[
#!if js then
print("ok!")
#!else
print("wrong!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if ..  elif", function()
      parser:parse([[
#!if nop then
print("wrong!")
#!else if js then
print("ok!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if ..  elif .. else", function()
      parser:parse([[
#!if nop then
print("wrong!")
#!else if alsoNop then
print("wrong!")
#!else
print("ok!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if not js then - without else", function()
      parser:parse([[
#!if not js then
print("wrong!")
#!end
]])
      assert(trans:next()):eq("\n")
    end)

    test("if not js then - with else", function()
      parser:parse([[
#!if not js then
print("wrong!")
#!else
print("ok!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if js then - without else", function()
      parser:parse([[
#!if js then
print("ok!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if py then - with else", function()
      parser:parse([[
#!if py then
print("wrong!")
#!else
print("ok!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if py then - without else", function()
      parser:parse([[
#!if py then
print("wrong!")
#!end
]])
      assert(trans:next()):eq("\n")
    end)

    test("if not py then - with else", function()
      parser:parse([[
#!if not py then
print("ok!")
#!else
print("wrong!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)

    test("if not py then - without else", function()
      parser:parse([[
#!if not py then
print("ok!")
#!end
]])
      assert(trans:next()):eq('print("ok!");\n')
    end)
  end)

  -------------
  -- runWith --
  -------------
  suite("runWith", function()
    test("/usr/bin/env node", function()
      parser:parse("#!/usr/bin/env node")
      assert(trans:next()):eq('#!/usr/bin/env node\n')
    end)
  end)

  ------------
  -- insert --
  ------------
  test("insert", function()
    parser:parse("#!insert /* istanbul ignore next */")
    assert(trans:next()):eq("/* istanbul ignore next */\n")
  end)
end):tags("directive")
